@@ -7,7 +7,7 @@ Tagged block data structure.
 
 """
 
-from __future__ import absolute_import, unicode_literals
+from __future__ import annotations
 
 import io
 import logging
@@ -633,6 +633,17 @@ class ProtectedSetting(IntegerElement):
     def position(self):
         return bool(self.value & 0x04)
 
+    @property
+    def nesting(self):
+        return bool(self.value & 0x08)
+
+    @property
+    def complete(self):
+        return self.value == 2147483648
+
+    def lock(self, lock_flags):
+        self.value = int(lock_flags)
+
 
 @register(Tag.REFERENCE_POINT)
 @attr.s(repr=False, slots=True)
@@ -703,7 +714,9 @@ class SheetColorSetting(ValueElement):
     .. py:attribute:: value
     """
 
-    value = attr.ib(default=0, converter=SheetColorType, type=SheetColorType)
+    value = attr.ib(
+        default=SheetColorType.NO_COLOR, converter=SheetColorType, type=SheetColorType
+    )
 
     @classmethod
     def read(cls, fp, **kwargs):
